package com.persagy.dmp.middleware.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.middleware.constant.CommonConsant;
import com.persagy.dmp.middleware.constant.enums.ClassCodeEnum;
import com.persagy.dmp.middleware.constant.enums.RelationTypeEnum;
import com.persagy.dmp.middleware.constant.enums.WdDefTypeEnum;
import com.persagy.dmp.middleware.constant.enums.WdModeCodeEnum;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.entity.vo.WdDefineTypeRel;
import com.persagy.dmp.middleware.entity.vo.WdObjectCount;
import com.persagy.dmp.middleware.service.*;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import com.persagy.dmp.middleware.utils.SortUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * @ClassName FloorServiceImpl
 * @Description: 楼层
 * @Author linhuili
 * @Date 2021/9/9 10:24
 * @Version V1.0
 **/
@Service
public class FloorServiceImpl implements IFloorService {

    @Autowired
    private IBaseService baseService;
    
    @Autowired
    private ISpaceService spaceService;

    @Autowired
    private IEquipService equipService;

    @Autowired
    private IWdDefineTypeRelService iWdDefineTypeRelService;

    @Autowired
    private IObjectCountService objectCountService;

    /**
     * 统计楼层下各信息
     * @param context
     * @param request
     * @return
     */
    @Override
    public CommonResult count(InstanceUrlParam context, CommonRequestDTO request) throws Exception {
        List<JSONObject> result = new ArrayList<>();
        JSONObject content = new JSONObject();

        //1. 查询楼层统计信息
        WdObjectCount wdObjectCount = new WdObjectCount();
        wdObjectCount.setBuildId(request.getBuildId());
        wdObjectCount.setFloorId(request.getFloorId());
        List<WdObjectCount> query = objectCountService.query(context, wdObjectCount);

        //2. 获取楼层基本信息
        getFloorBaseInfo(context,request,content);
        //处理统计数量
        handleFloorBaseCountNum(request,query,content);

        //3.查询楼层下的空间信息
        List<JSONObject> spaceList = queryFloorSpaceList(context, request);
        //处理统计数量
        handleFloorCountNum(request,query,spaceList);
        //类型名称相同的合并处理
        spaceList = EntityTransferUtil.combineData(spaceList);
        //按首字母，对象数量为0放后的规则进行排序
        spaceList = SortUtil.sortByCharacter(spaceList, CommonConsant.OBJ_TYPE, CommonConsant.CLASS_NAME);
        content.put("spaceInfos",spaceList);

        //4.返回结果
        result.add(content);
        return CommonResult.success(content);
    }


    /**
     *  处理楼层基本信息
     * @param requestDTO
     * @param query
     * @param spaceList
     */
    @Override
    public void handleFloorCountNum(CommonRequestDTO requestDTO,List<WdObjectCount> query,List<JSONObject> spaceList){
        if(CollUtil.isEmpty(query) || CollUtil.isEmpty(spaceList)){
            return;
        }
        if(StringUtils.isEmpty(requestDTO.getFloorId())){
            //处理全楼层的
            handleBuildCount(requestDTO,query,spaceList);
            return;
        }
        //处理单层的
        handleSingleFloorCount(requestDTO,query,spaceList);
    }


    /**
     * 处理单楼层统计情况
     * @param requestDTO
     * @param query
     * @param spaceList
     */
    private void handleBuildCount(CommonRequestDTO requestDTO,List<WdObjectCount> query,List<JSONObject> spaceList){
        if(CollUtil.isEmpty(query)){
            return;
        }

        //获取全局楼层统计情况，<buildId+"-"+systemCode+"-"+"bimTypeId",全集>
        Map<String, List<WdObjectCount>> countMap = getWdObjectCount(query);
        if(CollUtil.isEmpty(countMap)){
            return;
        }
        String buildId = requestDTO.getBuildId();
        for (JSONObject object : spaceList) {
            String systemCode = object.getString("systemCode");
            String bimTypeId = object.getString("bimTypeId");
            List<WdObjectCount> wdObjectCounts = countMap.get(buildId + "-" + systemCode + "-" + bimTypeId);
            if(CollUtil.isEmpty(wdObjectCounts)){
                continue;
            }
            AtomicInteger allNum = new AtomicInteger();
            wdObjectCounts.forEach(item->{
                Integer count = item.getCount();
                if(count == null){
                    return;
                }
                allNum.addAndGet(count.intValue());

            });
            object.put("count", allNum == null ? 0 : allNum);
        }
    }

    /**
     * 获取万达统计情况
     * @param query
     * @return
     */
    private Map<String, List<WdObjectCount>>  getWdObjectCount(List<WdObjectCount> query){
        Map<String, List<WdObjectCount>> countMap = new HashMap<>();
        query.forEach(item->{
            if(CollUtil.isEmpty(countMap)){
                List<WdObjectCount> list = new ArrayList<>();
                list.add(item);
                countMap.put(item.getBuildId()+"-"+item.getSystemCode()+"-"+item.getBimTypeId(),list);
                return;
            }
            List<WdObjectCount> wdObjectCounts = countMap.get(item.getBuildId() + "-" + item.getSystemCode() + "-" + item.getBimTypeId());
            if(CollUtil.isEmpty(wdObjectCounts)){
                wdObjectCounts = new ArrayList<>();
            }
            wdObjectCounts.add(item);
            countMap.put(item.getBuildId()+"-"+item.getSystemCode()+"-"+item.getBimTypeId(),wdObjectCounts);
        });
        return countMap;
    }

    /**
     * 处理单楼层统计情况
     * @param requestDTO
     * @param query
     * @param spaceList
     */
    private void handleSingleFloorCount(CommonRequestDTO requestDTO,List<WdObjectCount> query,List<JSONObject> spaceList){
        if(CollUtil.isEmpty(query)){
            return;
        }
        Map<String, WdObjectCount> countMap = new HashMap<>();
        query.forEach(item->{
            countMap.put(item.getBuildId()+"-"+item.getFloorId()+"-"+item.getSystemCode()+"-"+item.getBimTypeId(),item);
        });
        String buildId = requestDTO.getBuildId();
        String floorId = requestDTO.getFloorId();
        for (JSONObject object : spaceList) {
            String systemCode = object.getString("systemCode");
            String bimTypeId = object.getString("bimTypeId");
            WdObjectCount wdObjectCount = countMap.get(buildId + "-" + floorId + "-" + systemCode + "-" + bimTypeId);
            if(wdObjectCount == null){
                continue;
            }
            object.put("count",wdObjectCount.getCount());
        }
    }

    /**
     *  处理楼层基本信息
     * @param requestDTO
     * @param query
     * @param content
     */
    private void handleFloorBaseCountNum(CommonRequestDTO requestDTO,List<WdObjectCount> query,JSONObject content){
        if(CollUtil.isEmpty(query)){
            return;
        }
        if(StringUtils.isEmpty(requestDTO.getFloorId())){
            //处理全楼层的，暂时不需要统计
            return;
        }
        //处理单层的
        singleFloorBaseCount(requestDTO,query,content);
    }

    /**
     * 处理单楼层统计情况
     * @param requestDTO
     * @param query
     * @param content
     */
    private void singleFloorBaseCount(CommonRequestDTO requestDTO,List<WdObjectCount> query,JSONObject content){
        if(CollUtil.isEmpty(query)){
            return;
        }
        Map<String, WdObjectCount> countMap = new HashMap<>();
        query.forEach(item->{
            countMap.put(item.getBuildId()+"-"+item.getFloorId()+"-"+item.getSystemCode()+"-"+item.getBimTypeId(),item);
        });

        //楼层面积
        WdObjectCount wdObjectCount = countMap.get(requestDTO.getBuildId() + "-" + requestDTO.getFloorId() + "-LC-buildArea");
        if(wdObjectCount!= null){
            content.put("buildArea",wdObjectCount.getArea());
        }
        //租赁面积
        WdObjectCount wdObjectCountLease = countMap.get(requestDTO.getBuildId() + "-" + requestDTO.getFloorId() + "-LC-leaseArea");
        if(wdObjectCountLease!= null){
            content.put("leaseArea",wdObjectCountLease.getArea());
        }
        //该楼层各商铺数量
        WdObjectCount wdObjectCountStoreNum = countMap.get(requestDTO.getBuildId() + "-" + requestDTO.getFloorId() + "-LC-storeNum");
        if(wdObjectCountStoreNum!= null){
            content.put("storeNum",wdObjectCountStoreNum.getCount());
        }
       //统计该楼层机动车位数量
        WdObjectCount wdObjectCountCarNum = countMap.get(requestDTO.getBuildId() + "-" + requestDTO.getFloorId() + "-LC-countParkingCar");
        if(wdObjectCountCarNum!= null){
            content.put("countParkingCar",wdObjectCountCarNum.getCount());
        }
    }

    /**
     * 获取楼层基本信息
     * @param context
     * @param request
     * @param content
     * @throws Exception
     */
    private void getFloorBaseInfo(InstanceUrlParam context, CommonRequestDTO request,JSONObject content) throws Exception{
        //查询楼层信息
        List<JSONObject> floorList = queryFloorInfo(context, request);
        if(CollUtil.isEmpty(floorList)){
            throw new Exception("楼层不存在");
        }
        JSONObject floor = floorList.get(0);

        //统计楼层面积
        content.put("buildArea",floor.getDouble(CommonConsant.FLOOR_AREA));
        //统计租赁面积,默认返回0，不作初始化统计-20220330需求改
        //content.put("leaseArea",floor.getDouble(CommonConsant.FLOOR_LEASE_AREA));
        content.put("leaseArea",0);
        //统计该楼层各商铺数量
        Long storeNum = countStoreNum(context, request);
        content.put("storeNum",storeNum == null ? 0 : storeNum);

        //统计该楼层机动车位数量
        Long parkingCarNum = countParkingCar(context, request);
        content.put("countParkingCar",parkingCarNum == null ? 0 : parkingCarNum);
    }

    /**
     * 查询广联达楼层下的设备，空间，阀门
     * @param context
     * @param request
     * @return
     */
    @Override
    public List<JSONObject> query(InstanceUrlParam context, CommonRequestDTO request) {
        //查询楼层下的设备，空间，阀门
        List<JSONObject> result = new ArrayList<>();
        //查询楼层下的空间信息
        List<JSONObject> spaceList = queryFloorSpaceList(context, request);
        if(CollUtil.isNotEmpty(spaceList)){
            result.addAll(spaceList);
        }
        //查询楼层下的设备
        List<JSONObject> equipList = queryFloorEquipList(context, request);
        if(CollUtil.isNotEmpty(equipList)){
            result.addAll(equipList);
        }
        //查询楼层下的阀门
        List<JSONObject> materiaList = queryFloorMateriaList(context, request);
        if(CollUtil.isNotEmpty(materiaList)){
            result.addAll(materiaList);
        }
        return result;
    }

    /**
     * 查询广联达楼层下的空间
     * @param context
     * @param request
     * @return
     */
    @Override
    public List<JSONObject> queryFloorUnSpaceList(InstanceUrlParam context, CommonRequestDTO request) {
        //查询楼层下的空间
        List<JSONObject> result = new ArrayList<>();
        //查询楼层下的空间信息
        List<JSONObject> spaceList = queryFloorSpaceList(context, request);
        if(CollUtil.isNotEmpty(spaceList)){
            result.addAll(spaceList);
        }
        return result;
    }
    /**
     * 获取楼层下的空间
     * @param context
     * @param request
     */
    private List<JSONObject>  queryFloorSpaceList(InstanceUrlParam context, CommonRequestDTO request){
        //查询广联达空间类型
        WdDefineTypeRel defineTypeRel = new WdDefineTypeRel();
        defineTypeRel.setObjType(WdDefTypeEnum.SPACE.getValue());
        defineTypeRel.setWdModelCode(WdModeCodeEnum.LC.getValue());
        if(StringUtils.isNotEmpty(request.getSystemCode())){
            defineTypeRel.setWdSystemCode(request.getSystemCode());
        }
        List<WdDefineTypeRel> defineTypeRels = iWdDefineTypeRelService.query(context,defineTypeRel);

        //过滤不需要的类型，只需要楼层功能相关的空间
        //List<WdDefineTypeRel> floorDefineTypeRels = defineTypeRels.stream().filter(k->"LC".equals(k.getWdModelCode())).collect(Collectors.toList());

        //根据广联达空间类型查询空间信息
        List<JSONObject> spaceList = spaceService.querySpaceByGldTypeRel(context, defineTypeRels, request);
        return spaceList;
    }


    /**
     * 获取楼层下的设备
     * @param context
     * @param request
     */
    private List<JSONObject>  queryFloorEquipList(InstanceUrlParam context, CommonRequestDTO request){
        //查询广联达设备类型
        WdDefineTypeRel defineTypeRel = new WdDefineTypeRel();
        defineTypeRel.setObjType(WdDefTypeEnum.EQUIPMENT.getValue());
        List<WdDefineTypeRel> defineTypeRels = iWdDefineTypeRelService.query(context,defineTypeRel);

        //根据广联达设备类型查询设备信息
        List<JSONObject> equipList = equipService.queryEquipByGldTypeRel(context, defineTypeRels, request);
        return equipList;
    }

    /**
     * 获取楼层下的设备
     * @param context
     * @param request
     */
    public List<JSONObject>  queryFloorMateriaList(InstanceUrlParam context, CommonRequestDTO request){
        //查询广联达设备类型
        WdDefineTypeRel defineTypeRel = new WdDefineTypeRel();
        defineTypeRel.setObjType(WdDefTypeEnum.MATERIAL.getValue());
        List<WdDefineTypeRel> defineTypeRels = iWdDefineTypeRelService.query(context,defineTypeRel);

        //根据广联达设备类型查询阀门信息
        List<JSONObject> materiaList = equipService.queryMateriaByGldTypeRel(context, defineTypeRels, request);
        return materiaList;
    }

    /**
     * 查询楼层信息
     * @param context
     * @param requestDTO
     * @return
     */
    private List<JSONObject> queryFloorInfo(InstanceUrlParam context, CommonRequestDTO requestDTO){
        ObjectNode jsonNodes = JsonNodeFactory.instance.objectNode();
        jsonNodes.put("id",requestDTO.getFloorId());
        QueryCriteria criteria = new QueryCriteria();
        criteria.setCriteria(jsonNodes);

        CommonResult response = baseService.doQuery(context, criteria, JSONObject.class);
        return (List<JSONObject>)response.getContent();
    }


    /**
     * 统计该楼层机动车位数量
     * @param context
     * @param request
     * @return
     */
    private Long countParkingCar(InstanceUrlParam context, CommonRequestDTO request){
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        criteria.put(CommonConsant.CLASS_CODE, ClassCodeEnum.WEGMMP.getValue());
        QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria,RelationTypeEnum.MECHINARCH_EQ2FL.getGraphCode(),
                RelationTypeEnum.MECHINARCH_EQ2FL.getRelCode(), request.getFloorId(), 1);
        CommonResult commonResult = baseService.doQuery(context, queryCriteria, JSONObject.class);
       return commonResult.getCount();
    }


    /**
     * 统计商铺数量
     * @param context
     * @param request
     * @return
     */
    private Long countStoreNum(InstanceUrlParam context, CommonRequestDTO request){
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        criteria.put(CommonConsant.ROOM_LOCAL_FUNC_TYPE_ID, CommonConsant.STORE_BIMTYPEID);
        QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria,RelationTypeEnum.ARCHSUBSET_FL2SP.getGraphCode(),
                RelationTypeEnum.ARCHSUBSET_FL2SP.getRelCode(), request.getFloorId(), 0);
        CommonResult commonResult = baseService.doQuery(context, queryCriteria, JSONObject.class);
        return commonResult.getCount();
    }

}
