package com.persagy.dmp.middleware.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.dmp.common.constant.CommonConstant;
import com.persagy.dmp.middleware.dao.WdObjectCountMapper;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.vo.WdObjectCount;
import com.persagy.dmp.middleware.service.IObjectCountService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 对象统计保存
 *
 * @author:linhuili
 * @date:2022/4/20
 */
@Service
public class ObjectServiceImpl extends ServiceImpl<WdObjectCountMapper, WdObjectCount>  implements IObjectCountService {

    @Autowired
    private WdObjectCountMapper dao;

    /**
     * 保存统计信息
     * @param wdObjectCounts
     */
    @Override
    public void save(InstanceUrlParam context,List<WdObjectCount> wdObjectCounts) {
        //创建默认条件
        createDefaultValue(context,wdObjectCounts);

        List<WdObjectCount> result = new ArrayList<>();
        for (WdObjectCount objectCount : wdObjectCounts) {
            List<WdObjectCount> content = queryWdObjectCount(objectCount);
            if(CollUtil.isNotEmpty(content)){
                WdObjectCount wdObjectCount = content.get(0);
                if(objectCount.getCount()!=null){
                    wdObjectCount.setCount(objectCount.getCount());
                }
                if(objectCount.getArea()!=null){
                    wdObjectCount.setArea(objectCount.getArea());
                }
                result.add(wdObjectCount);
            }else {
                result.add(objectCount);
            }
        }
        saveOrUpdateBatch(result);
    }

    /**
     * 查询万达统计数量
     * @param context
     * @param wdObjectCount
     * @return
     */
    @Override
    public List<WdObjectCount> query(InstanceUrlParam context, WdObjectCount wdObjectCount) {
        wdObjectCount.setGroupCode(context.getGroupCode());
        wdObjectCount.setProjectId(context.getProjectId());
        return queryWdObjectCount(wdObjectCount);
    }

    /**
     * 查询wd统计数量
     * @param objectCount
     * @return
     */
    private List<WdObjectCount> queryWdObjectCount(WdObjectCount objectCount){
        QueryWrapper<WdObjectCount> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("group_code",objectCount.getGroupCode());
        queryWrapper.eq("project_id",objectCount.getProjectId());
        queryWrapper.eq("build_id",objectCount.getBuildId());
        if(StringUtils.isNotEmpty(objectCount.getFloorId())){
            queryWrapper.eq("floor_id",objectCount.getFloorId());
        }
        if(StringUtils.isNotEmpty(objectCount.getSystemCode())){
            queryWrapper.eq("system_code",objectCount.getSystemCode());
        }
        if(StringUtils.isNotEmpty(objectCount.getBimTypeId())){
            queryWrapper.eq("bim_type_id",objectCount.getBimTypeId());
        }
        queryWrapper.eq("valid",1);
       return dao.selectList(queryWrapper);
    }


    /**
     * 创建默认条件
     * @param voList
     */
    private void createDefaultValue(InstanceUrlParam context,List<WdObjectCount> voList){
        String groupCode = context.getGroupCode();
        if(StringUtils.isEmpty(groupCode)){
            //默认集团
            groupCode = CommonConstant.DEFAULT_ID;
        }
        String projectId = context.getProjectId();
        if(StringUtils.isEmpty(projectId)){
            //默认项目
            projectId = CommonConstant.DEFAULT_ID;
        }
        for (WdObjectCount vo : voList) {
            vo.setGroupCode(groupCode);
            vo.setProjectId(projectId);
        }
    }
}
